'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
const tslib_1 = require('tslib');
const fs_1 = tslib_1.__importDefault(require('fs'));
const path_1 = tslib_1.__importDefault(require('path'));
const shelljs_1 = tslib_1.__importDefault(require('shelljs'));
const utils_1 = require('../utils/index');
const worker_1 = tslib_1.__importDefault(require('../utils/worker'));
const chalk_1 = tslib_1.__importDefault(require('chalk'));
const ora_1 = tslib_1.__importDefault(require('ora'));
class Schedule {
    initConfig(standardConfig) {
        this.standardConfig = standardConfig;
    }
    run() {
        this.overridesEslint();
        // 写入 eslint 配置文件
        this.overridesPrettier();
        // 写入 prettier 配置文件
        this.updatePackageJson();
        // 更新 package.json 配置文件
        this.installDependencies();
        // 安装相关依赖
        this.addHusky();
        // 添加 commit 自动检测
        this.autoFix();
        // 自动将项目中不规范的代码格式化
        this.addCommit();    // 将本次规范改动自动创建一个 commit 记录（防止自动检测触发回滚）
    }
    overridesEslint() {
        const standardConfig = this.standardConfig.getEslintConfig();
        const filePath = path_1.default.join(worker_1.default.getWorkDir(), './.eslintrc.js');
        fs_1.default.writeFileSync(filePath, `module.exports = ${ JSON.stringify(standardConfig, null, 2) };`);
    }
    overridesPrettier() {
        const prettierConfig = this.standardConfig.getPrettierConfig();
        const filePath = path_1.default.join(worker_1.default.getWorkDir(), './.prettierrc.js');
        fs_1.default.writeFileSync(filePath, `module.exports = ${ JSON.stringify(prettierConfig, null, 2) };`);
    }
    updatePackageJson() {
        const pkPath = path_1.default.join(worker_1.default.getWorkDir(), './package.json');
        const pkConfig = this.getPackageJsonConfig();
        fs_1.default.writeFileSync(pkPath, JSON.stringify(pkConfig, null, 2));
    }
    getPackageJsonConfig() {
        const pkConfig = this.standardConfig.getPackageJsonConfig();
        const pkPath = path_1.default.join(worker_1.default.getWorkDir(), './package.json');
        const currentPkConfig = JSON.parse(fs_1.default.readFileSync(pkPath, 'utf-8'));
        const config = utils_1.merge(currentPkConfig, pkConfig);
        // 清除某些被设置在 dependencies 中的 eslint 相关依赖
        const dependencies = config.dependencies;
        const devDependencies = config.devDependencies;
        for (const key in dependencies) {
            if (dependencies[key] && devDependencies[key]) {
                delete dependencies[key];
            }
        }
        return config;
    }
    installDependencies() {
        const spinner = ora_1.default(chalk_1.default.blue('配置文件更新完成\uFF0C正在安装相关依赖\uFF0C请耐心等待...')).start();
        const reply = shelljs_1.default.exec('npm install --only=dev');
        spinner.stop();
        if (reply.code !== 0) {
            shelljs_1.default.exit(1);
        }
    }
    addHusky() {
        shelljs_1.default.exec('npx husky install && npx husky set .husky/pre-commit "npm run jgxl-pre-commit"');
    }
    autoFix() {
        if (!this.standardConfig.getAutoFix())
            return;
        shelljs_1.default.exec('eslint --fix ./src');
    }
    addCommit() {
        shelljs_1.default.exec('git add .');
        shelljs_1.default.exec('git commit -m "add standard by jgxl-cli" --no-verify');
    }
}
exports.default = Schedule;