import fs from 'fs';
import path from 'path';
import shell from 'shelljs';
import { merge } from '@/utils';
import worker from '@/utils/worker';
import Configuration from './configuration';
import chalk from 'chalk';
import ora from 'ora';

export default class Schedule {
  private standardConfig!: Configuration;

  public initConfig(standardConfig: Configuration) {
    this.standardConfig = standardConfig;
  }

  public run() {
    this.overridesEslint(); // 写入 eslint 配置文件
    this.overridesPrettier(); // 写入 prettier 配置文件
    this.updatePackageJson(); // 更新 package.json 配置文件
    this.installDependencies(); // 安装相关依赖
    this.addHusky(); // 添加 commit 自动检测
    this.autoFix(); // 自动将项目中不规范的代码格式化
    this.addCommit(); // 将本次规范改动自动创建一个 commit 记录（防止自动检测触发回滚）
  }

  private overridesEslint() {
    const standardConfig = this.standardConfig.getEslintConfig();
    const filePath = path.join(worker.getWorkDir(), './.eslintrc.js');
    fs.writeFileSync(filePath, `module.exports = ${JSON.stringify(standardConfig, null, 2)};`);
  }

  private overridesPrettier() {
    const prettierConfig = this.standardConfig.getPrettierConfig();
    const filePath = path.join(worker.getWorkDir(), './.prettierrc.js');
    fs.writeFileSync(filePath, `module.exports = ${JSON.stringify(prettierConfig, null, 2)};`);
  }

  private updatePackageJson() {
    const pkPath = path.join(worker.getWorkDir(), './package.json');
    const pkConfig = this.getPackageJsonConfig();
    fs.writeFileSync(pkPath, JSON.stringify(pkConfig, null, 2));
  }

  private getPackageJsonConfig() {
    const pkConfig = this.standardConfig.getPackageJsonConfig();
    const pkPath = path.join(worker.getWorkDir(), './package.json');
    const currentPkConfig = JSON.parse(fs.readFileSync(pkPath, 'utf-8'));
    const config = merge(currentPkConfig, pkConfig);

    // 清除某些被设置在 dependencies 中的 eslint 相关依赖
    const dependencies = config.dependencies;
    const devDependencies = config.devDependencies;
    for (const key in dependencies) {
      if (dependencies[key] && devDependencies[key]) {
        delete dependencies[key];
      }
    }
    return config;
  }

  private installDependencies() {
    const spinner = ora(chalk.blue('配置文件更新完成，正在安装相关依赖，请耐心等待...')).start();
    const reply = shell.exec('npm install --only=dev');
    spinner.stop();
    if (reply.code !== 0) {
      shell.exit(1);
    }
  }

  private addHusky() {
    shell.exec('npx husky install && npx husky set .husky/pre-commit "npm run jgxl-pre-commit"');
  }

  private autoFix() {
    if (!this.standardConfig.getAutoFix()) return;

    shell.exec('eslint --fix ./src');
  }

  private addCommit() {
    shell.exec('git add .');
    shell.exec('git commit -m "add standard by jgxl-cli" --no-verify');
  }
}
